@@ -59,11 +59,11 @@ App::Dochazka::REST::Model::Privhistory - privilege history functions
 
 =head1 VERSION
 
-Version 0.207
+Version 0.252
 
 =cut
 
-our $VERSION = '0.207';
+our $VERSION = '0.252';
 
 
 
@@ -180,6 +180,19 @@ our @EXPORT_OK = qw( get_privhistory );
 
 =head1 METHODS
 
+
+=head2 expurgate
+
+Non-destructively convert object into hashref
+
+=cut
+
+sub expurgate {
+    my ( $self ) = @_;
+    return App::Dochazka::REST::Model::Shared::expurgate( $self );
+}
+
+
 =head2 load_by_eid
 
 Supposed to be a class method, but in reality we just don't care what the first
@@ -279,68 +292,27 @@ sub delete {
 
 =head2 get_privhistory
 
-Given an EID and an optional tsrange, return the history of privilege level
-changes for that employee over the given tsrange, or the entire history if no
-tsrange is supplied. Returns a status object where the payload is a reference
-to an array of C<privhistory> objects. If nothing is found, the array will be
-empty. If there is a DBI error, the payload will be undefined.
+Takes a PARAMHASH which can have one or more of the properties 'eid', 'nick',
+and 'tsrange'.
+
+At least one of { 'eid', 'nick' } must be specified. If both are specified,
+the employee is determined according to 'eid'.
+
+The function returns the history of privilege level changes for that employee
+over the given tsrange, or the entire history if no tsrange is supplied. 
+
+The return value will always be an L<App::CELL::Status|status> object.
+
+Upon success, the payload will contain a 'history' key, the value of which will
+be a reference to an array of C<privhistory> objects. If nothing is found, the
+array will be empty. If there is a DBI error, the payload will be undefined.
 
 =cut
 
 sub get_privhistory {
-    validate_pos( @_, 1, 1, 0, 0, 0, 0 );
-    my %ARGS = validate( @_, { 
-        eid => { type => SCALAR, optional => 1 },
-        nick => { type => SCALAR, optional => 1 },
-        tsrange => { type => SCALAR, optional => 1 },
-    } );
-
-    my $tsr = ( exists $ARGS{'tsrange'} )
-        ? $ARGS{'tsrange'}
-        : '[,)';
-        
-    my ( $sql, $sk, $status, $result );
-    if ( exists $ARGS{'nick'} ) {
-        $sql = $site->SQL_PRIVHISTORY_SELECT_RANGE_BY_NICK;
-        $result->{'nick'} = $ARGS{'nick'};
-        $sk = $ARGS{'nick'};
-    }
-    if ( exists $ARGS{'eid'} ) {
-        $sql = $site->SQL_PRIVHISTORY_SELECT_RANGE_BY_EID;
-        $result->{'eid'} = $ARGS{'eid'};
-        $sk = $ARGS{'eid'};
-    }
-    die "AAAAAAAAAAAHHHHH! Engulfed by the abyss" unless $sk and $sql and $tsr;
-
-    my $counter = 0;
-    $dbh->{RaiseError} = 1;
-    try {
-        my $sth = $dbh->prepare( $sql );
-        $sth->execute( $sk, $tsr );
-        while( defined( my $tmpres = $sth->fetchrow_hashref() ) ) {
-            $counter += 1;
-            push @{ $result->{'privhistory'} }, $tmpres;
-        }
-    } catch {
-        my $arg = $dbh->err
-            ? $dbh->errstr
-            : $_;
-        $status = $CELL->status_err( 'DOCHAZKA_DBI_ERR', args => [ $arg ] );
-    };
-    $dbh->{RaiseError} = 0;
-    return $status if defined $status;
-    if ( $counter > 0 ) {
-        $status = $CELL->status_ok( 'DISPATCH_RECORDS_FOUND', args => 
-            [ $counter ], payload => $result, count => $counter );
-    } else {
-        $result->{'privhistory'} = [];
-        $status = $CELL->status_ok( 'DISPATCH_NO_RECORDS_FOUND', 
-            payload => $result, count => $counter );
-    }
-    $dbh->{RaiseError} = 0;
-    return $status;
+    return App::Dochazka::REST::Model::Shared::get_history( 'priv', @_ );
 }
-  
+
 
 
 
